#-----------------------------------------------------------------------------
# Name:        MessageTool.py
# Purpose:     
#
# Author:      Philipp Auersperg
#
# Created:     2003/10/01
# RCS-ID:      $Id: MessageTool.py,v 1.4 2003/09/25 14:54:04 azy Exp $
# Copyright:   (c) 2003 BlueDynamics
# Licence:     GPL
#-----------------------------------------------------------------------------

import Globals
from DateTime import DateTime

from Globals import HTMLFile, InitializeClass
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_inner, aq_parent

from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import CMFCorePermissions

from Products.CMFMessage.Message import Message
from Products.CMFPlone.PloneFolder import PloneFolder

class MessageTool(  UniqueObject, SimpleItem):

    meta_type = 'CMF Message Tool'
    id = 'portal_messagetool'
    
    security = ClassSecurityInfo()

    def findUser(self,uid):
        ''' finds a user through all acl_user folders'''
        
        uf=self.acl_users
        while(uf):
            u=uf.getUserById(uid)
            if u:
                return u.__of__(uf)
            
            try:
                uf=uf.aq_parent.aq_parent.acl_users
            except AttributeError:
                return None
    
    def sendPersonalMessages(self,recipient,REQUEST=None,**kw):
        ''' allow bulk messages'''
        if REQUEST:
            kw.update(REQUEST.form)
    
        del kw['recipient']
    
        if type(recipient) in [type(''),type(u'')]:
            #if recipients are a array expr as string, eval them
            try:
                recipients=eval(recipient)
            except:
                #then they are a comma separated string
                recipients=recipient.split(',')
        else:
            recipients=recipient    

        ## try managing user names

        try:
            gtool = getToolByName(self,'portal_groups')
            g_list = gtool.listGroupIds()
            recipients_with_groups = []
            for r in recipients:
                if r in g_list:
                    u_list = gtool.getGroupById(r).getGroupUsers()
                    recipients_with_groups.extend([str(u) for u in u_list])
                else:
                    recipients_with_groups.append(r)

            recipients = recipients_with_groups
            __traceback_info__ = recipients
        except AttributeError:
            pass
        
        for r in recipients:
            self.sendPersonalMessage(recipient=r,REQUEST=REQUEST,**kw)
        
        
    def sendPersonalMessage( self, id=None, subject='', text='', recipient=None
                   , text_format='html', REQUEST=None , nextURL='.', sendFromUrl=0, fromUrl=None, **kw):
        """
            Add a NewsItem
        """
        now=DateTime()
        recipient=recipient.strip()
        
        if not recipient:
            if REQUEST:
                return REQUEST.RESPONSE.redirect('message_send_form')
    #            return REQUEST.RESPONSE.redirect(REQUEST['HTTP_REFERRER'])
            else:
                raise ValueError,'empty recipient'
        
        if REQUEST:
            username=str(REQUEST['AUTHENTICATED_USER'])
        else:
            username='unknown'
            
        #if not id:
        id='from '+username+'.'+now.strftime('%Y-%m-%d')+'.'+now.strftime('%H_%M_%S')
        rec=self.findUser(recipient)
    
        #and now get the UserFolder
        if recipient:
            membership=getToolByName(self,'portal_membership')
            pers=membership.getPersonalFolder(recipient)
            catalog = getToolByName(self, 'portal_catalog')
            
            # stole that from CMFPlone.MembershipTool 
            # more elegant would be a createPersonalFolder method there
            # but for backward compatibility i cant rely on it...
            if not pers:
                member_folder=membership.getHomeFolder(recipient)

                if member_folder is None:
                    raise  AttributeError, 'The User (%s) has no member folder' % recipient
                
                member_folder.manage_addPloneFolder(membership.personal_id, 'Personal Items')

                personal=getattr(member_folder, membership.personal_id)
                personal.description = "contains personal workarea items for %s" % recipient
                personal.changeOwnership(rec)
                personal.manage_setLocalRoles(recipient, ['Owner'])
            
                catalog.unindexObject(personal) #dont add .personal folders to catalog
                pers=personal
            
            if 'messages' not in pers.objectIds():
                #pers.invokeFactory(id='messages', title='Personal Messages', type_name='Folder')
                ff=PloneFolder('messages','Personal Messages')
                pers._setObject('messages',ff)
                dest=getattr(pers,'messages')
                dest.changeOwnership(rec)
                catalog.unindexObject(dest) #dont add .personal folders to catalog
                
            dest=getattr(pers,'messages')
        else:
            dest=self
            
        if sendFromUrl:
            #fromUrl=self.absolute_url()
            #print 'Message::send : ',fromUrl,self
            pass
        else:
            fromUrl=None
            
        dest.invokeFactory(id=id, type_name='Message'
                  , subject=subject
                  , text=text
                  , description=text
                  , recipient=recipient
                  , sender=username
                  , text_format=text_format
                  , fromUrl=fromUrl
                  )
    
        o=getattr(dest, id, None)
        o.changeOwnership(rec)
    
        if REQUEST:
            REQUEST.RESPONSE.redirect(nextURL)
    

InitializeClass( MessageTool )
