#-----------------------------------------------------------------------------
# Name:        UserTrack.py
# Purpose:     
#
# Author:      Philipp Auersperg
#
# Created:     2003/10/01
# RCS-ID:      $Id: UserTrack.py,v 1.1.1.1 2003/04/03 01:08:54 zworkb Exp $
# Copyright:   (c) 2003 BlueDynamics
# Licence:     GPL
#-----------------------------------------------------------------------------
from ZPublisher.HTTPRequest import BaseRequest
from OFS.SimpleItem import Item
from OFS.PropertyManager import PropertyManager
import Acquisition
from time import time
import os.path
from Globals import DTMLFile

from Acquisition import aq_inner,aq_parent

originalTraverse=BaseRequest.traverse
userdata={}

instance_userdata={}

#for these classes use the aq_parent as starter
classesToUseParent=['ZopePageTemplate','FSPageTemplate','DTMLMethod','FSDTMLMethod'] 

def setUserInfo(self,user,path):
    ''' '''
    username=str(user)
    userdata[username]={'last_request': time(), 'path': path, 'user':user, 'username':username}
    
    #find the right place to start with searching
    try:
        if self.__class__.__name__ in  classesToUseParent:
            par=self.aq_parent
        else:
            par=self
    except:
        par=None

    if not par:
        return

    ut=getattr(par,'portal_activeusers',None)

    while ut: #get the parent instances too
        ut.setUserInfo(user,path)
        if not ut.doPromote():
            break
        par=aq_parent(aq_inner(ut))
        if par:
            ut=getattr(par.aq_parent,'portal_activeusers',None)
        else:
            ut=None

def listGlobalUserActivities(self):
    ''' lists Zope-wide user activities'''
    res = userdata.copy()
    for k in res.keys():
        res[k]['time_elapsed'] = time() - res[k]['last_request']
        
    return res

def wrappedTraverse(self, path, response=None, validated_hook=None):
    ''' wraps BaseRequest::traverse in order to
    trace authenticated User's activities
    for example to get a list of currently active users
    self is a BaseRequest
    '''

    #print 'wrappedTraverse before:',path # ,self.keys()
    res=originalTraverse(self,path,response,validated_hook)
    
    t=time()
    if os.path.splitext(path)[-1].lower() not in ('.jpg','.gif','.png','.js','.class','.css'):
        #avoid calling setUserInfo for all non-page requests
        setUserInfo(res,self['AUTHENTICATED_USER'],path)
    #print 'ready:', time() - t
    return res
    
def installWrapper():
    '''installs the wrapper into BaseRequest.traverse '''
    
    BaseRequest.traverse=wrappedTraverse
    #print 'installed traverse Wrapper'
    
    
#-----------------------------------

class UserTrack(Item, Acquisition.Implicit, PropertyManager):
    ''' User tracker - tracks the presence of
        authenticated users in a volatile way
    '''
    
    timeout=300.0
    list_anon=0
    promote=1
    
    #_v_userdata = {}
    _properties=(
        {'id':'timeout','type':'float'},
        {'id':'list_anon','type':'boolean'},
        {'id':'promote','type':'boolean'}, #promote the logging to aq_parent
        )
    
    manage_options=(
        {'label':'view','action':'index_html'},
        ) + PropertyManager.manage_options

    def __init__(self,id='portal_activeusers', title='Who is active in the portal?'):
        #print 'track::__init__:id',id
        self.id=id
        self.title=title
        
    def setUserInfo(self,user,path):
        ''' sets the userinfo locally for this tracker_instance '''
        
        #if not self.__dict__.has_key('_v_userdata'):
        #    self._v_userdata={}
        username=str(user)

        if self.list_anon or username!='Anonymous User':
            self.userdata()[username]={'last_request': time(), 'path': path, 'user':user, 'username':username}
        
    def userdata(self):
        id=self.absolute_url()
        if not instance_userdata.has_key(id):
            instance_userdata[id]={}
        return instance_userdata[id]
    
        return self._v_userdata
    
    def isUserOnline(self, r, timeout):
        ''' checks based on the record if a user is online 
            can be overridden by other implementations
            the default implementation just checks if he requested
            anything below the portal_activeusers object
            within timeout seconds
        '''
        
        return time() - r['last_request'] < timeout
        
    def listUserActivities(self, timeout=None):
        ''' lists local user activities'''
        
        
        if timeout is None:
            timeout=self.timeout
            
        res = [r for r in self.userdata().values() if self.isUserOnline(r, timeout)]
        
        for k in res:
           k['time_elapsed'] = time() - k['last_request']
            
        return res
        
    index_html=DTMLFile('dtml/index_html',globals())
    
    def doPromote(self):
        ''' shal logging promoted to the next
        portal_activeusers instance in aq_parent?'''
        
        return getattr(self,'promote',1)

        
def manage_addUserTrack(self):
    """ creates a usertracker """
    
    track = UserTrack()
    #print 'track.id',track.id
    self._setObject(track.getId(),track)
    
    